
#include "fccdb.h"
#include "fcc_contour.h"
#include <cassert>
#include <cstring>

void do_station(Facility& fac, Fm_Eng_Data& ed, int count)
{
      std::string callsign = (fac.fac_callsign == "NEW") 
	? fac.facility_id 
	: fac.fac_callsign;
      std::string out_file_name = ed.station_channel + '-' + callsign 
	+ '-' + char(count + '0') + '-' + ed.fm_dom_status + ".contour";
      std::cout << out_file_name << '\n';
      std::ofstream out(out_file_name.c_str());
      
      double max_power = 
	std::max(std::max(to_double(ed.horiz_erp), to_double(ed.vert_erp)),
		 std::max(to_double(ed.max_horiz_erp), to_double(ed.max_vert_erp)));
      double haat =
	std::max(to_double(ed.max_haat),
		 std::max(to_double(ed.haat_horiz_rc_mtr), to_double(ed.haat_vert_rc_mtr)));
      
      out << "#" << fac.fac_callsign << ' ' << fac.comm_city << "," << fac.comm_state << ' '
	  << ed.fm_dom_status << ' ' << fac.facility_id << ' ' << ed.antenna_id << '\n'
	  << ed.lat_deg << ' ' << ed.lat_min << ' ' << ed.lat_sec << ' ' << ed.lat_dir << ' '
	  << ed.lon_deg << ' ' << ed.lon_min << ' ' << ed.lon_sec << ' ' << ed.lon_dir << ' '
	  << max_power << ' ' << ed.station_channel << '\n'
	  << "bearing\thaat\tfield\tdb\tpower\tp60\ti40\ti54\ti100\n";
      
      if (ed.antenna_type != "N" && ed.antenna_id != "") {
	std::ifstream f3("ant_pattern.dat");
	while (f3) {
	  Ant_Pattern ap(f3);
	  if (ed.antenna_id == ap.antenna_id) {
	    double field = to_double(ap.field_value);
	    double db = 20*log10(field);
	    double power = max_power * pow(10., db/10.);
	    out.precision(3);
	    out << to_double(ap.azimuth) << '\t' << haat << '\t' 
		<< field << '\t' << db << '\t' << power << '\t'
		<< contour_distance(power, haat, 60., PROTECTED, 211) << '\t'
		<< contour_distance(power, haat, 40., INTERFERING, 211) << '\t'
		<< contour_distance(power, haat, 54., INTERFERING, 211) << '\t'
		<< contour_distance(power, haat, 100., INTERFERING, 211) << '\n';
	  }else{
	  }
	}		  
      }else{
	// non-directional antenna
	double power = max_power;
	for (double azimuth = 0.; azimuth < 360.; azimuth += 10.) {
	  out.precision(3);
	  out << azimuth << '\t' << haat << '\t' 
	      << 1.0 << '\t' << 0. << '\t' << power << '\t'
	      << contour_distance(power, haat, 60., PROTECTED, 211) << '\t'
	      << contour_distance(power, haat, 40., INTERFERING, 211) << '\t'
	      << contour_distance(power, haat, 54., INTERFERING, 211) << '\t'
	      << contour_distance(power, haat, 100., INTERFERING, 211) << '\n';
	}
      }
      out << '\n';

}


int main(int argc, char* argv[])
{
  {
    std::ifstream f1("facility.dat");
    
    if (argc == 1) {
      while (f1) {
	Facility d(f1);
	d.dump();
      }
    }else if (argv[1][0] == '-') {
      std::string latitude[10], longitude[10], channel[10], state[10];
      int i = 1;
      while (i < argc) {
	if (strncmp(argv[i], "-la", 3) == 0) {
	  ++i;
	  int j=0;
	  while (i < argc && argv[i][0] != '-' && j < 10) {
	    latitude[j++] = argv[i++];
	  }
	}else if (strncmp(argv[i], "-lo", 3) == 0) {
	  ++i;
	  int j=0;
	  while (i < argc && argv[i][0] != '-' && j < 10) {
	    longitude[j++] = argv[i++];
	  }
	}else if (strncmp(argv[i], "-c", 2) == 0) {
	  ++i;
	  int j=0;
	  while (i < argc && argv[i][0] != '-' && j < 10) {
	    channel[j++] = argv[i++];
	  }
	}else if (strncmp(argv[i], "-s", 2) == 0) {
	  ++i;
	  int j=0;
	  while (i < argc && argv[i][0] != '-' && j < 10) {
	    state[j++] = argv[i++];
	  }
	}else{
	  std::cout << "bad arg: " << argv[i] << '\n';
	}
      }
      while (f1) {
	Facility fac(f1);
	bool state_matches  = (state[0] == "" || state[0] == fac.comm_state);
	for (int i = 1; !state_matches && i < 10 && state[i] != ""; ++i) {
	  state_matches = (state[i] == fac.comm_state);
	}
	if (state_matches && fac.fac_service == "FM") {
	  std::cout << fac.fac_callsign << '\n';
	  std::ifstream f2("fm_eng_data.dat");
	  int count = 0;
	  while (f2) {
	    Fm_Eng_Data ed(f2);
	    if (fac.facility_id == ed.facility_id) {
	      ++count;
	      bool channel_matches = (channel[0] == "" || channel[0] == ed.station_channel);
	      for (int i = 1; !channel_matches && i < 10 && channel[i] != ""; ++i) {
		channel_matches = (channel[i] == ed.station_channel);
	      }
	      bool latitude_matches = (latitude[0] == "" || latitude[0] == ed.lat_deg);
	      for (int i = 1; !latitude_matches && i < 10 && latitude[i] != ""; ++i) {
		latitude_matches = (latitude[i] == ed.lat_deg);
	      }
	      bool longitude_matches = (longitude[0] == "" || longitude[0] == ed.lon_deg);
	      for (int i = 1; !longitude_matches && i < 10 && longitude[i] != ""; ++i) {
		longitude_matches = (longitude[i] == ed.lon_deg);
	      }
	      if (channel_matches && latitude_matches && longitude_matches) {
		do_station(fac, ed, count);
	      }else{
	      }
	    }else{
	    }
	  }
	}
      }
    }else{
      while (f1) {
	Facility fac(f1);
	if ((argv[1] == fac.facility_id) 
	    || (fac.fac_callsign.compare(0, strlen(argv[1]), argv[1]) == 0)) {
	  std::ifstream f2("fm_eng_data.dat");
	  int count = 0;
	  while (f2) {
	    Fm_Eng_Data ed(f2);
	    if (fac.facility_id == ed.facility_id) {
	      ++count;
	      do_station(fac, ed, count);
	    }else{
	    }
	  }
	}else{
	}
      }
    }
  }
  return 0;
}
