/* $Id$ -*- C++ -*-
 */

#include "contour.h"
#include <cstring>
#include "fcc_contour.h"

void fix_contours(Station* s)
{
  integer channel = s->_channel;

  for (Station::iterator i = s->begin(); i != s->end(); ++i) {
    i->_p60 = contour_distance(i->_power, i->_haat, 60., PROTECTED, s->_channel);
    i->_i40 = contour_distance(i->_power, i->_haat, 40., INTERFERING, s->_channel);
    i->_i54 = contour_distance(i->_power, i->_haat, 54., INTERFERING, s->_channel);
    i->_i100 = contour_distance(i->_power, i->_haat, 100., INTERFERING, s->_channel);
  }
}

void find_area(Station* s)
{
  double area_p60 = 0;
  double area_i40 = 0;
  double area_i54 = 0;
  double area_i100 = 0;

  Station::iterator i = s->end()-1;
  double r1_p60 = i->_p60;
  double r1_i40 = i->_i40;
  double r1_i54 = i->_i54;
  double r1_i100 = i->_i100;
  Angle b1 = i->_angle;

  for (i = s->begin(); i != s->end(); ++i) {    
    Angle b2 = i->_angle;
    double a = ((b2-b1).dbl())/360;
    b1 = b2;

    double r2_p60 = i->_p60;
    double r = (r2_p60 + r1_p60) / 2;
    area_p60 += a * M_PI * r * r;
    r1_p60 = r2_p60;
    
    double r2_i40 = i->_i40;
    r = (r2_i40 + r1_i40) / 2;
    area_i40 += M_PI * a * r * r;
    r1_i40 = r2_i40;

    double r2_i54 = i->_i54;
    r = (r2_i54 + r1_i54) / 2;
    area_i54 += M_PI * a * r * r;
    r1_i54 = r2_i54;

    double r2_i100 = i->_i100;
    r = (r2_i100 + r1_i100) / 2;
    area_i100 += M_PI * a * r * r;
    r1_i100 = r2_i100;
  }

  std::cout << '\n';
  std::cout.setf(std::ios_base::fixed, std::ios_base::floatfield);
  std::cout.precision(0);
  std::cout << "area"
      << '\t'
      << '\t'
      << '\t'
      << '\t'
      << '\t' << area_p60
      << '\t' << area_i40
      << '\t' << area_i54
      << '\t' << area_i100
      << '\n';
}

int main(int argc, char* argv[])
{
  Station* s = NULL;
  int arg = 1;
  if (argc > 1 && argv[1][0] != '-') {
    std::ifstream f(argv[1]);
    s = new Station(f);
    ++arg;
  }else{
    s = new Station(std::cin);
  }

  int increment = 1;
  for (int i = arg; i < argc; i += increment) {
    increment = 1;
    if (strcmp(argv[i], "-fix-haat") == 0) {
      increment = 2;
      for (Station::iterator si = s->begin(); si != s->end(); ++si) {
	double adjust = atof(argv[i+1]);
	si->_haat += adjust;
      }
      // don't fix contours.  leaves an inconsistent state.
    }else if (strcmp(argv[i], "-field") == 0) {
      for (Station::iterator si = s->begin(); si != s->end(); ++si) {
	si->_antenna_rel_db = 20 * log10(si->_antenna_rel_field);
	si->_power = s->_power * pow(10, si->_antenna_rel_db/10);
      }
      fix_contours(s);
    }else if (strcmp(argv[i], "-db") == 0) {
      for (Station::iterator si = s->begin(); si != s->end(); ++si) {
	si->_antenna_rel_field = pow(10, si->_antenna_rel_db/20);
	si->_power = s->_power * pow(10, si->_antenna_rel_db/10);
      }
      fix_contours(s);
    }else if (strcmp(argv[i], "-power") == 0) {
      for (Station::iterator si = s->begin(); si != s->end(); ++si) {
	si->_antenna_rel_db = 10 * log10(si->_power / s->_power);
	si->_antenna_rel_field = pow(10, si->_antenna_rel_db/20);
      }
      fix_contours(s);
    }else if (strcmp(argv[i], "-p60") == 0) {
      integer channel = s->_channel;
      for (Station::iterator si = s->begin(); si != s->end(); ++si) {
	real erp = si->_power;
	real haat = si->_haat;
	real distance = si->_p60;
	integer iswitch = 1; // compute field from distance;
	char flag[] = "      ";
	integer curve = 0;   // F(50,50)
	real field = NA;
	for (int ii=0; ii<2; ++ii) {
	  tvfmfs_metric_(&erp, &haat, &channel, &field, &distance, &iswitch, &curve, flag);
	  double adjustment = 60 - field;
	  si->_antenna_rel_db += adjustment;
	  erp = si->_power = s->_power * pow(10, si->_antenna_rel_db/10);
	}
	si->_antenna_rel_field = pow(10, si->_antenna_rel_db/20);
      }
      fix_contours(s);
    }else if (strcmp(argv[i], "-i40") == 0) {
      integer channel = s->_channel;
      for (Station::iterator si = s->begin(); si != s->end(); ++si) {
	real erp = si->_power;
	real haat = si->_haat;
	real distance = si->_i40;
	integer iswitch = 1; // compute field from distance;
	char flag[] = "      ";
	integer curve = 1;   // F(50,10)
	real field = NA;
	for (int ii=0; ii<2; ++ii) {
	  tvfmfs_metric_(&erp, &haat, &channel, &field, &distance, &iswitch, &curve, flag);
	  double adjustment = 40 - field;
	  si->_antenna_rel_db += adjustment;
	  erp = si->_power = s->_power * pow(10, si->_antenna_rel_db/10);
	}
	si->_antenna_rel_field = pow(10, si->_antenna_rel_db/20);
      }
      fix_contours(s);
    }else if (strcmp(argv[i], "-i54") == 0) {
      integer channel = s->_channel;
      for (Station::iterator si = s->begin(); si != s->end(); ++si) {
	real erp = si->_power;
	real haat = si->_haat;
	real distance = si->_i54;
	integer iswitch = 1; // compute field from distance;
	char flag[] = "      ";
	integer curve = 1;   // F(50,10)
	real field = NA;
	for (int ii=0; ii<2; ++ii) {
	  tvfmfs_metric_(&erp, &haat, &channel, &field, &distance, &iswitch, &curve, flag);
	  double adjustment = 54 - field;
	  si->_antenna_rel_db += adjustment;
	  erp = si->_power = s->_power * pow(10, si->_antenna_rel_db/10);
	}
	si->_antenna_rel_field = pow(10, si->_antenna_rel_db/20);
      }
      fix_contours(s);
    }else{
      std::cout << "error: " << argv[i] << " unrecognized\n";
    }
  }

  s->write(std::cout);
  find_area(s);

  return 0;
}
